// Fichier libre de droits

#include "ArgumentsLigneDeCommande.h"
#include "ImplementationListe.h"
#include "ImplementationBit.h"
#include "ContenuFichier.h"
#include "Solitaire.h"
#include "ImplementationBit.hpp"

ArgumentsLigneDeCommande::ArgumentsLigneDeCommande(int argc, const char *argv[]) {
	programmeFilename = argv[0];
	askHelp = false;
	askCopyright = false;
	explorationComplete = false;
	forceImplementationAvecListe = false;
	utiliserDictionnaire = true;
	allowHexagonal = false;
	erreur = false;
	for (int i = 1; i < argc; ++i) {
		if ('-' == argv[i][0]) {
			if ( ('\0' != argv[i][1]) && ('\0' == argv[i][2]) ) {
				switch (argv[i][1]) {
				case 'H' :
				case 'h' :
					askHelp = true;;
					break;
				case 'v' :
				case 'V' :
					askCopyright = true;
					break;
				case 'f' :
				case 'F' :
					if ( (i + 1 < argc) && (argv[i+1][0] != '-') ) {
						fichiersDemandes.push_back(argv[i+1]);
						// L'argument suivant est trait maintenant
						i += 1;
					} else {
						std::cerr << "Argument manquant : le nom du fichier doit suivre '-f'\n";
						erreur = true;
					}
					break;
				case 'l' :
				case 'L' :
					forceImplementationAvecListe = true;
					break;
				case 'e' :
				case 'E' :
					explorationComplete = true;
					break;
				case 'x' :
				case 'X' :
					allowHexagonal = true;
					break;
				case 'd' :
				case 'D' :
					utiliserDictionnaire = false;
					break;
				default :
					std::cerr << "Argument inconnu : " << argv[i] << "\n";
					erreur = true;
					break;
				}
			} else {
				std::cerr << "Argument inconnu : " << argv[i] << "\n";
				erreur = true;
			}
		} else {
			std::cerr << "Argument inconnu : " << argv[i] << "\n";
			erreur = true;
		}
	}
	if (erreur) {
		std::cerr << "\nDans le doute, ne rien faire.\n" << std::endl;
	}
}

void ArgumentsLigneDeCommande::display(std::ostream &os) {
	os << "askHelp : " << askHelp << "\n";
	os << "askCopyright : " << askCopyright << "\n";
	os << "erreur : " << erreur << "\n";
	os << "explorationComplete : " << explorationComplete << "\n";
	os << "forceImplementationAvecListe : " << forceImplementationAvecListe << "\n";
	os << "utiliserDictionnaire : " << utiliserDictionnaire << "\n";
	os << "fichiersDemandes : " << "\n";
	for (std::list<const char *>::const_iterator i = fichiersDemandes.begin(); i != fichiersDemandes.end(); ++i) {
		os << "\t" << *i << "\n";
	}
}

void ArgumentsLigneDeCommande::displayHelp(void) const {
	std::cout << "\nSyntaxe :\n" << programmeFilename << " [-h] [-v] [-filename]\n"
		<< "-h: Affiche cette aide\n-v: Affiche des informations de copyright\n"
		<< "-f: Lit un plateau de jeu dans le fichier filename\n"
		<< "\t (les options suivantes sont hors concours mais pratiques)\n"
		<< "-l: Force l'utilisation des listes comme description interne du plateau de jeu\n\t(probablement plus lent, mais utile pour tester)\n"
		<< "-e: Demande l'exploration de toutes les configurations\n\t(pour compter les solutions et les impasses)\n\tPour les petits terrains de jeu et/ou les grosses machines\n"
		<< "-d: Ne pas retenir les configurations parcourues pendant la recherche\n\tOn utilise ainsi un strict minimum de ressources machine\n\tmais en contrepartie la recherche est beaucoup plus longue\n"
		<< "-x: autorise les plateaux dont les cases sont hexagonales.\n\tLe fichier de description du plateau doit alors commencer par la ligne \"hexa\"\n\t et les lignes se poussent d'une demi-case vers la droite en descendant.\n"
		<< "\n(Parler de l'encodage du plateau)\n"
		<< "Bon d'accord : on code une case vide avec un '.', une case contenant un pion avec 'x'\n"
		<< "et on utilise ' ' pour ce qui n'est pas une case. Voir les fichiers exemples.\n"
		<< "On n'autorise qu'une seule case vide, et on impose que\n"
		<< "le dernier pion finisse sur cette case"
		<< std::endl;
}

void ArgumentsLigneDeCommande::displayCopyright(void) const {
	std::cout << "Copyright : RAS\nC'est tout libre et tout et tout." << std::endl;
}

int ArgumentsLigneDeCommande::exec(void) const {
	if (erreur) {
		displayHelp();
		return EXIT_FAILURE;
	}
	if (askHelp) {
		displayHelp();
		return EXIT_FAILURE;
	}
	if (askCopyright) {
		displayCopyright();
		return EXIT_FAILURE;
	}
	if (fichiersDemandes.size() == 0) {
		displayHelp();
		return EXIT_FAILURE;
	}
	if (fichiersDemandes.size() > 1) {
		std::cerr << "Je ne traite qu'un seul fichier, car :\n"
			"- On ne m'a pas dit qu'on devait pouvoir en traiter plusieurs\n"
			"- Il ne faut pas abuser de ce que je sais faire :\n    rien qu'un seul fichier, c'est potentiellement long\n"
			"(oui, je pourrais au moins m'occuper du premier, mais une erreur est une erreur.\n"
			"Et comme le chantait Coluche :\n\"Soit faignant, soit faignant, l'avenir t'attend.\")" << std::endl;
		return EXIT_FAILURE;
	}
	// Boucle pour la forme, puisque il y a eu une erreur si il n'y a pas exactement un fichier  ouvrir.
	for (std::list<const char *>::const_iterator fileToOpen = fichiersDemandes.begin(); fileToOpen != fichiersDemandes.end(); ++fileToOpen) {
		ContenuFichier lePlateauDeJeu(*fileToOpen, allowHexagonal);
		if (1 != lePlateauDeJeu.casesLibres.size()) {
			std::cerr << "Fichier non valable :\n"
				"on impose qu'il n'y ait qu'une seule case vide\n"
				"(qui en plus sera obligatoirement la case finale)" << std::endl;
			return EXIT_FAILURE;
		}
		if ( forceImplementationAvecListe) {
			etudeSolitaire<ImplementationListe>(lePlateauDeJeu);
		} else if (lePlateauDeJeu.colonneMaximum < ImplementationBit<unsigned char>::getMaximumSize()) {
			etudeSolitaire<ImplementationBit<unsigned char> >(lePlateauDeJeu);
		} else if (lePlateauDeJeu.colonneMaximum < ImplementationBit<unsigned short int>::getMaximumSize()) {
			etudeSolitaire<ImplementationBit<unsigned short int> >(lePlateauDeJeu);
		} else if (lePlateauDeJeu.colonneMaximum < ImplementationBit<unsigned long int>::getMaximumSize()) {
			etudeSolitaire<ImplementationBit<unsigned long int> >(lePlateauDeJeu);
			// Ne passe pas le -pedantic de gcc :spamafote:
			//        } else if (lePlateauDeJeu.colonneMaximum < ImplementationBit<unsigned long long int>::getMaximumSize()) {
			//            etudeSolitaire<ImplementationBit<unsigned long long int> >(lePlateauDeJeu);
		} else {
			etudeSolitaire<ImplementationListe>(lePlateauDeJeu);
		}
	}
	return EXIT_SUCCESS;
}

template<class Implementation>
void ArgumentsLigneDeCommande::etudeSolitaire(const ContenuFichier &lePlateauDeJeu) const {
	Solitaire<Implementation> jeu(lePlateauDeJeu);
	if (explorationComplete) {
		jeu.explore(std::cout);
		jeu.afficheExploration(std::cout);
	} else {
		if (jeu.solve(utiliserDictionnaire)) {
			jeu.afficheSolution(std::cout);
			if (afficheTaillesDictionnaires) {
				jeu.afficheDictionnaires(std::cout);
			}
		} else {
			std::cout << "Pas de solution" << std::endl;
		}
	}
}
