// Fichier libre de droits

#include <string>
#include "ImplementationListe.h"
#include "Exceptions.h"
#include "Mouvement.h"

ImplementationListe::Point::Point(int lig, int col) {
	ligne = lig;
	colonne = col;
}

void ImplementationListe::Point::display(std::ostream &os) const {
	os << "(" << ligne << ", " << colonne << ")";
}

bool ImplementationListe::Point::operator<(const Point &pl) const {
	if (ligne < pl.ligne) return true;
	if (ligne > pl.ligne) return false;
	return (colonne < pl.colonne);
}

bool ImplementationListe::Point::operator!=(const Point &pl) const {
	return ( (ligne != pl.ligne) || (colonne != pl.colonne) );
}

ImplementationListe::ImplementationListe() {
	isHexagonal = false;
}

ImplementationListe::ImplementationListe(const ImplementationListe &il) {
	listeMouvements = il.listeMouvements;
	desc = il.desc;
	isHexagonal = il.isHexagonal;
}

ImplementationListe &ImplementationListe::operator=(const ImplementationListe &il) {
	if (this != &il) {
		ImplementationListe temp(il);
		swap(temp);
	}
	return *this;
}

void ImplementationListe::swap(ImplementationListe &il) {
	listeMouvements.swap(il.listeMouvements);
	desc.swap(il.desc);
	bool tempBool = il.isHexagonal;
	il.isHexagonal = isHexagonal;
	isHexagonal = tempBool;
}

bool ImplementationListe::libre(const Point &p) const {
	return ( desc.end() == desc.find(p) );
}

void ImplementationListe::retire(const Point &p) {
	desc.erase(p);
}

void ImplementationListe::ajoute(const Point &p) {
	desc.insert(p);
}

int ImplementationListe::taille(void) const {
	// cast pour viter un warning
	// On cache un problme de dpassement si la taille du plateau de jeu dpasse la taille d'un entier.
	// Je suis donc plutt confiant...
	return static_cast<int>(desc.size());
}

void ImplementationListe::display(std::ostream &os, const char *caractereOccupe) const {
	std::string caseLibre(" ");
	std::string caseOccupee(caractereOccupe);
	if (isHexagonal) {
		caseLibre += " ";
		caseOccupee += " ";
	}
	std::set<Point>::const_iterator pointsIterator = desc.begin();
	if (desc.end() == pointsIterator) {
		return;
	}
	int colonne = 0;
	int ligne = 0;
	while (desc.end() != pointsIterator) {
		while (ligne < pointsIterator->ligne) {
			os << '\n';
			colonne = 0;
			ligne += 1;
		}
		if (0 == colonne) {
			if (isHexagonal) {
				os << std::string(ligne, ' ');
			}
		}
		while (colonne < pointsIterator->colonne) {
			os << caseLibre;
			colonne += 1;
		}
		os << caseOccupee;
		colonne += 1;
		pointsIterator++;
	}
	os << '\n';
}

void ImplementationListe::initListeMouvements(void) {
	listeMouvements.clear();
	std::set<Point>::const_iterator iteratorPoint1 = desc.begin();
	if (desc.end() == iteratorPoint1) {
		return;
	}
	std::set<Point>::const_iterator iteratorPoint2 = iteratorPoint1;
	iteratorPoint2++;
	if (desc.end() == iteratorPoint2) {
		return;
	}
	std::set<Point>::const_iterator iteratorPoint3 = iteratorPoint2;
	iteratorPoint3++;
	while (desc.end() != iteratorPoint3) {
		if ( (iteratorPoint1->ligne == iteratorPoint3->ligne) && (2 == iteratorPoint3->colonne - iteratorPoint1->colonne) ) {
			Assert(iteratorPoint1->ligne == iteratorPoint2->ligne);
			Assert(iteratorPoint1->colonne + 1 == iteratorPoint2->colonne);
			// test simplifi, car on connait l'ordre utilis dans le std::set<Point>
			listeMouvements.push_back(Mouvement<Point>(*iteratorPoint1, *iteratorPoint2, *iteratorPoint3));
			listeMouvements.push_back(Mouvement<Point>(*iteratorPoint3, *iteratorPoint2, *iteratorPoint1));
		}
		iteratorPoint1++;
		iteratorPoint2++;
		iteratorPoint3++;
	}
	for (iteratorPoint1 = desc.begin(); iteratorPoint1 != desc.end(); ++iteratorPoint1) {
		iteratorPoint3 = desc.find(Point(iteratorPoint1->ligne + 2, iteratorPoint1->colonne));
		if ( iteratorPoint3 != desc.end()) {
			iteratorPoint2 = desc.find(Point(iteratorPoint1->ligne + 1, iteratorPoint1->colonne));
			if ( iteratorPoint2 != desc.end()) {
				listeMouvements.push_back(Mouvement<Point>(*iteratorPoint1, *iteratorPoint2, *iteratorPoint3));
				listeMouvements.push_back(Mouvement<Point>(*iteratorPoint3, *iteratorPoint2, *iteratorPoint1));
			}
		}
	}
	if (isHexagonal) {
		for (iteratorPoint1 = desc.begin(); iteratorPoint1 != desc.end(); ++iteratorPoint1) {
			iteratorPoint3 = desc.find(Point(iteratorPoint1->ligne + 2, iteratorPoint1->colonne - 2));
			if ( iteratorPoint3 != desc.end()) {
				iteratorPoint2 = desc.find(Point(iteratorPoint1->ligne + 1, iteratorPoint1->colonne - 1));
				if ( iteratorPoint2 != desc.end()) {
					listeMouvements.push_back(Mouvement<Point>(*iteratorPoint1, *iteratorPoint2, *iteratorPoint3));
					listeMouvements.push_back(Mouvement<Point>(*iteratorPoint3, *iteratorPoint2, *iteratorPoint1));
				}
			}
		}
	}
}

bool ImplementationListe::operator==(const ImplementationListe &d) const {
	// D'aprs mon algorithme, la comparaison ne doit avoir lieu qu'entre des listes de mme taille
	Assert(desc.size() == d.desc.size());
	Assert(isHexagonal == d.isHexagonal);
	std::set<Point>::const_iterator i = desc.begin();
	std::set<Point>::const_iterator j = d.desc.begin();
	while (i != desc.end()) {
		Assert(j != d.desc.end());
		if (*i != *j) {
			return false;
		}
		++i;
		++j;
	}
	Assert(j == d.desc.end());
	return true;
}

bool ImplementationListe::operator<(const ImplementationListe &d) const {
	// D'aprs mon algorithme, la comparaison ne doit avoir lieu qu'entre des listes de mme taille
	Assert(desc.size() == d.desc.size());
	Assert(isHexagonal == d.isHexagonal);
	std::set<Point>::const_iterator i = desc.begin();
	std::set<Point>::const_iterator j = d.desc.begin();
	while (i != desc.end()) {
		if (*i != *j) {
			return (*i < *j);
		}
		++i;
		++j;
	}
	Assert(j == d.desc.end());
	return false;
}

void ImplementationListe::init(bool hexa, int) {
	isHexagonal = hexa;
}
